package main

import (
	_ "embed"
	"flag"
	"log"
	"math/rand"
	"os"
	"text/template"
	"time"
)

//go:embed beretta.json
var defconfig string

type RepoUpdate struct {
	repo Repo
	Tag  string
}

func notifierThread(c chan RepoUpdate, n Notifier) {
	update := <-c
	log.Printf("New tag stored for %s: %s", update.repo.IdTuple(), update.Tag)
	if n.Notify(update.repo, update.Tag) {
		log.Printf("Notified for %s: %s", update.repo.IdTuple(), update.Tag)
	}
	notifierThread(c, n)
}

func repoThread(c chan RepoUpdate, repo Repo, avgInterval int, db Db) {
	log.Println("Checking: ", repo.IdTuple())
	tag, err := repo.GetLatestTag()
	isNewVersion, err_2 := db.CheckAndStore(repo.IdTuple(), tag.Name)

	if err != nil {
		log.Printf("Failed to fetch latest tag for %s: %v", repo.IdTuple(), err)
	} else if err_2 != nil {
		log.Printf("Failed to store latest tag for %s: %v", repo.IdTuple(), err_2)
	} else if isNewVersion {
		c <- RepoUpdate{repo, tag.Name}
	}
	log.Println("Checking done: ", repo.IdTuple())

	sleepTime := time.Duration(rand.Intn(avgInterval)) * time.Second
	time.Sleep(sleepTime)

	repoThread(c, repo, avgInterval, db)
}

func main() {
	conf_path := flag.String("config", "beretta.json", "The path to the config file")
	db_path := flag.String("db", "cache.db", "The path to the cache database")
	flag.Parse()

	tmpl, err := template.New("config").Parse(defconfig)
	if err != nil {
		panic(err)
	}

	randtk, err := randToken(8)
	if err != nil {
		panic(err)
	}

	data := map[string]any{
		"TopicToken": randtk,
	}

	if _, err := os.Stat(*conf_path); os.IsNotExist(err) {
		log.Printf("Writing new config file: %s", *conf_path)
		file, err := os.Create(*conf_path)
		if err != nil {
			panic(err)
		}
		err = tmpl.Execute(file, data)
		if err != nil {
			log.Fatalf("Error writing config file: %s", err)
			return
		}
	}

	log.Printf("Using config path: %s", *conf_path)
	log.Printf("Using db path: %s", *db_path)

	// Load configuration
	config, err := loadConfig(*conf_path)
	if err != nil {
		log.Fatalf("Failed to load configuration: %v", err)
	}

	// Create database
	db, err := NewSQLiteDb(*db_path)
	if err != nil {
		log.Fatalf("Failed to create database: %v", err)
	}

	notifier := NtfyNotifier{Topic: config.NtfyTopic}
	log.Printf("Sending notifications to: %s", notifier.Url())

	c := make(chan RepoUpdate)

	// Spawn a goroutine for each repository
	for _, repo := range config.Repos {
		go repoThread(c, repo, config.Interval, &db)
	}

	notifierThread(c, notifier)
}
