package main

import (
	"encoding/json"
	"fmt"
	"log"
	"net/http"
)

type Repo interface {
	GetLatestTag() (Tag, error)
	TagsUrl() string
	Owner() string
	Name() string
	IdTuple() string
}

// Description of a repository (owner and name)
type RepoConfig struct {
	OwnerName string `json:"owner"`
	RepoName  string `json:"repo"`
}

func (r RepoConfig) GetLatestTag() (Tag, error) {
	// url := fmt.Sprintf("https://api.github.com/repos/%s/%s/tags", r.Owner, r.Url)

	resp, err := http.Get(r.TagsUrl())
	if err != nil {
		log.Printf("Failed to fetch releases for %s: %v", r.TagsUrl(), err)
		return Tag{}, err
	}
	defer resp.Body.Close()

	if resp.StatusCode != http.StatusOK {
		log.Printf("Unexpected response status for %s: %d", r.TagsUrl(), resp.StatusCode)
		return Tag{}, fmt.Errorf("unexpected response status: %d", resp.StatusCode)
	}

	var tags = []Tag{}
	if err := json.NewDecoder(resp.Body).Decode(&tags); err != nil {
		log.Printf("Failed to decode response for %s: %v", r.TagsUrl(), err)
		return Tag{}, err
	}

	return tags[0], nil
}

func (r RepoConfig) TagsUrl() string {
	return fmt.Sprintf("https://api.github.com/repos/%s/%s/tags", r.OwnerName, r.RepoName)
}

func (r RepoConfig) Owner() string {
	return r.OwnerName
}

func (r RepoConfig) Name() string {
	return r.RepoName
}

func (r RepoConfig) IdTuple() string {
	return fmt.Sprintf("%s/%s", r.OwnerName, r.RepoName)
}

type Commit struct {
	Sha string `json:"sha"`
	URL string `json:"url"`
}

type Tag struct {
	Name       string `json:"name"`
	ZipballURL string `json:"zipball_url"`
	TarballURL string `json:"tarball_url"`
	Commit     Commit `json:"commit"`
	NodeID     string `json:"node_id"`
}
